"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.showPopup = void 0;
var React = require("react");
var ReactDOM = require("react-dom");
var _1 = require(".");
var Types_1 = require("./Types");
/** @hidden @internal */
function showPopup(layoutDiv, triggerElement, items, onSelect, classNameMapper) {
    var currentDocument = triggerElement.ownerDocument;
    var triggerRect = triggerElement.getBoundingClientRect();
    var layoutRect = layoutDiv.getBoundingClientRect();
    var elm = currentDocument.createElement("div");
    elm.className = classNameMapper(Types_1.CLASSES.FLEXLAYOUT__POPUP_MENU_CONTAINER);
    if (triggerRect.left < layoutRect.left + layoutRect.width / 2) {
        elm.style.left = triggerRect.left - layoutRect.left + "px";
    }
    else {
        elm.style.right = layoutRect.right - triggerRect.right + "px";
    }
    if (triggerRect.top < layoutRect.top + layoutRect.height / 2) {
        elm.style.top = triggerRect.top - layoutRect.top + "px";
    }
    else {
        elm.style.bottom = layoutRect.bottom - triggerRect.bottom + "px";
    }
    _1.DragDrop.instance.addGlass(function () { return onHide(); });
    _1.DragDrop.instance.setGlassCursorOverride("default");
    layoutDiv.appendChild(elm);
    var onHide = function () {
        _1.DragDrop.instance.hideGlass();
        layoutDiv.removeChild(elm);
        ReactDOM.unmountComponentAtNode(elm);
        elm.removeEventListener("mousedown", onElementMouseDown);
        currentDocument.removeEventListener("mousedown", onDocMouseDown);
    };
    var onElementMouseDown = function (event) {
        event.stopPropagation();
    };
    var onDocMouseDown = function (event) {
        onHide();
    };
    elm.addEventListener("mousedown", onElementMouseDown);
    currentDocument.addEventListener("mousedown", onDocMouseDown);
    ReactDOM.render(React.createElement(PopupMenu, { currentDocument: currentDocument, onSelect: onSelect, onHide: onHide, items: items, classNameMapper: classNameMapper }), elm);
}
exports.showPopup = showPopup;
/** @hidden @internal */
var PopupMenu = function (props) {
    var items = props.items, onHide = props.onHide, onSelect = props.onSelect, classNameMapper = props.classNameMapper;
    var onItemClick = function (item, event) {
        onSelect(item);
        onHide();
        event.stopPropagation();
    };
    var itemElements = items.map(function (item, i) { return (React.createElement("div", { key: item.index, className: classNameMapper(Types_1.CLASSES.FLEXLAYOUT__POPUP_MENU_ITEM), "data-layout-path": "/popup-menu/tb" + i, onClick: function (event) { return onItemClick(item, event); }, title: item.node.getHelpText() }, item.node._getRenderedName())); });
    return (React.createElement("div", { className: classNameMapper(Types_1.CLASSES.FLEXLAYOUT__POPUP_MENU), "data-layout-path": "/popup-menu" }, itemElements));
};
//# sourceMappingURL=PopupMenu.js.map